/*   Author: Martijn van der Maarl
 *   Date: 04-19-2020
 *   Version: English
 *   
 *   This code is based on the code in the Word Clock from BasWage, so all the credits go to him. See the following URL:
 *   https://www.instructables.com/id/Word-Clock-RGB/
 *   I am definitely not a master in coding, so there are probably a few unnecessary things. Feel free to use this code and modify it! If you have any questions, feel free to leave a comment in my instructable. 
 */


#include <Adafruit_NeoPixel.h>
#include "Wire.h"

#include "DS3231.h"
#ifdef __AVR__
    #include <avr/power.h>
#endif

// Adafruit neopixel Pin                                                Op pin D6 zit de data-wire van de ledstrip
#define NEO_PIN 6

// Define modes                                                         Modes voor digitale / normaal, niet nodig. Voor buttons
#define NORMAL   0

int numberOfModes = 1;

//                                                                      Kleuren bepalen in de kleurenmodus, niet nodig dus
#define WHITE           0
int colorMode = WHITE;
int numberOfColorModes = 1;


// Define words                                                             Geeft aan welke leds moeten oplichten bij welke tijd
int minVijfOver[] = {0,1,2,-1};
int minTienOver[] = {3,4,5,6,-1};
int minKwartOver[] = {7,8,9,10,-1};
int minTienVoorHalf[] = {11,12,13,14,-1};
int minVijfVoorHalf[] = {15,16,17,18,-1};
int minHalf[] = {19,20,-1};
int minVijfOverHalf[] = {21,22,23,24,-1};
int minTienOverHalf[] = {25,26,27,28,-1};
int minKwartVoor[] = {29,30,31,32,-1};
int minTienVoor[] = {33,34,35,36,-1};
int minVijfVoor[] = {37,38,39,-1};

int Uur[] = {40,41,-1};

int uurTwaalf[] = {42,-1};
int uurElf[] = {43,-1};
int uurTien[] = {44,-1};
int uurNegen[] = {45,-1};
int uurAcht[] = {46,-1};
int uurZeven[] = {47,-1};
int uurZes[] = {48,-1};
int uurVijf[] = {49,-1};
int uurVier[] = {50,-1};
int uurDrie[] = {51,-1};
int uurTwee[] = {52,-1};
int uurEen[] = {53,-1};

int digitalPos1[] = {104,105,106,103,102,101,78,79,80,77,76,75,52,53,54};
int digitalPos2[] = {107,108,109,100,99,98,81,82,83,74,73,72,55,56,57};
int digitalColon[] = {97,71, -1};
int digitalPos3[] = {111,112,113,96,95,94,85,86,87,70,69,68,59,60,61};
int digitalPos4[] = {114,115,116,93,92,91,88,89,90,67,66,65,62,63,64};


int numberOfPixels = 54;                                                                          // hoeveel pixel zitten er in de ledstrip

// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(numberOfPixels, NEO_PIN, NEO_GRB + NEO_KHZ800);       // waar bestaat de NeoPixel strip uit? Dit klopt zo.

// Define colors                                                                                  Voor nu nog niet nodig (ALLEEN ROOD GEBRUIKEN VOOR TESTS), later kan dit gebruikt worden. 
uint32_t red = strip.Color(255, 0, 0);
uint32_t green = strip.Color(0, 255, 0);
uint32_t blue = strip.Color(0, 0, 255);
uint32_t yellow = strip.Color(255, 255, 0);                                             
uint32_t white = strip.Color(255, 255, 255);                                             
uint32_t off = strip.Color(0, 0, 0);

// Define colorgroups
uint32_t colorMinutes = off;
uint32_t colorHours = off;
uint32_t colorNotUsed = off;

// Daylight saving time
boolean DST = true;

// Used to force an update of the time
boolean triggerUpdate = false;

//                                                                                                    Gebruikt om te wisselen in kleur modus via buttons, niet nodig dus. 
void setColors() {
    switch(colorMode) {
        case WHITE:
            colorMinutes = white;
            colorHours = blue;
            colorNotUsed = off;
            break; 
        default:
            #ifdef SERIALON
            Serial.println("Unknown color mode, using WHITE!");
            #endif
            colorMode = WHITE;
            setColors();
            break;
    }
}

void setup() {
    Wire.begin(); // is dit nog nodig? volgens mij is dit nodig voor de RTC..
    #ifdef SERIALON
    Serial.begin(9600);
    #endif
    strip.begin();
    strip.show(); // Initialize all pixels to 'off'

    /* set the initial time here:
    * DS3231 seconds, minutes, hours, day of week (1=Sunday, 7=Saturday),
    * day of month (1 to 31), month, year (0 to 99)
    
   setDS3231time(0,1,18,5,16,4,20);
    */

     setColors();

    #ifdef SERIALON
    Serial.println("Setup complete!");
    #endif

}

//                                                                                                                Dit lijkt voor een random kleur modus, niet nodig dus.
void displayLeds(int pixels[], uint32_t color) {
    if(colorMode == WHITE) {
        color = strip.Color(255,255,255);
    }
    for( int i = 0; pixels[i] != -1; i++) {
        if(colorMode == WHITE) {
            color = strip.Color(255,255,255);
        }
        strip.setPixelColor(pixels[i], color);
    }
}

 
// Function that switches off all leds                                                            Function that switches off all leds
void ledsOff() {
    for (int pixel=0; pixel < numberOfPixels; pixel++){
        strip.setPixelColor(pixel, colorNotUsed);
    } 
}


void displayDigitalNumber(int position, int number) { 
    int *p, *n;
    uint32_t color;
    color = red;
    
    switch(position) {
        case 1:
            color = red;
            p = digitalPos1;
            break;
        case 2:
            color = blue;
            p = digitalPos2;
            break;
        case 3:
            color = red;
            p = digitalPos3;
            break;
        case 4:
            color = blue;
            p = digitalPos4;
            break;
        default: 
            break;        
    }

    switch (number) {
        case 0: { int t[] = {p[0], p[1], p[2], p[3], p[5], p[6], p[8], p[9], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 1: { int t[] = {p[1], p[4], p[7], p[10], p[13], -1}; displayLeds(t, color); break; }
        case 2: { int t[] = {p[0], p[1], p[2], p[5], p[6], p[7], p[8], p[9], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 3: { int t[] = {p[0], p[1], p[2], p[5], p[6], p[7], p[8], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 4: { int t[] = {p[0], p[2], p[3], p[5], p[6], p[7], p[8], p[11], p[14], -1}; displayLeds(t, color); break; }
        case 5: { int t[] = {p[0], p[1], p[2], p[3], p[6], p[7], p[8], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 6: { int t[] = {p[0], p[1], p[2], p[3], p[6], p[7], p[8], p[9], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 7: { int t[] = {p[0], p[1], p[2], p[5], p[8], p[11], p[14], -1}; displayLeds(t, color); break; }
        case 8: { int t[] = {p[0], p[1], p[2], p[3], p[5], p[6], p[7], p[8], p[9], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        case 9: { int t[] = {p[0], p[1], p[2], p[3], p[5], p[6], p[7], p[8], p[11], p[12], p[13], p[14], -1}; displayLeds(t, color); break; }
        default: break;
    }    
}



void displayDigitalTime(byte minute, byte hour) {
    ledsOff();

    // Hours
    displayDigitalNumber(1,hour/10); 
    displayDigitalNumber(2,hour%10); 
    
    // Minutes
    displayDigitalNumber(3,minute/10); 
    displayDigitalNumber(4,minute%10); 
    
    strip.show();
}


void displayTime(byte minute, byte hour) {
    ledsOff();
 //    displayItIs();
    // Minutes
    switch (minute) {
        case 0: displayLeds(Uur,colorMinutes); break;
        case 1: displayLeds(Uur,colorMinutes); break;
        case 2: displayLeds(Uur,colorMinutes); break; 
        case 3: displayLeds(Uur,colorMinutes); break;
        case 4: displayLeds(minVijfOver,colorMinutes); break;
        case 5: displayLeds(minVijfOver,colorMinutes); break;
        case 6: displayLeds(minVijfOver,colorMinutes); break;
        case 7: displayLeds(minVijfOver,colorMinutes); break;
        case 8: displayLeds(minVijfOver,colorMinutes); break;
        case 9: displayLeds(minTienOver,colorMinutes); break;
        case 10: displayLeds(minTienOver,colorMinutes); break;
        case 11: displayLeds(minTienOver,colorMinutes); break;
        case 12: displayLeds(minTienOver,colorMinutes); break;
        case 13: displayLeds(minTienOver,colorMinutes); break;
        case 14: displayLeds(minKwartOver,colorMinutes); break;
        case 15: displayLeds(minKwartOver,colorMinutes); break;
        case 16: displayLeds(minKwartOver,colorMinutes); break;
        case 17: displayLeds(minKwartOver,colorMinutes); break;
        case 18: displayLeds(minKwartOver,colorMinutes); break;
        case 19: displayLeds(minTienVoorHalf,colorMinutes); break;
        case 20: displayLeds(minTienVoorHalf,colorMinutes); break;
        case 21: displayLeds(minTienVoorHalf,colorMinutes); break;
        case 22: displayLeds(minTienVoorHalf,colorMinutes); break;
        case 23: displayLeds(minTienVoorHalf,colorMinutes); break;
        case 24: displayLeds(minVijfVoorHalf,colorMinutes); break;
        case 25: displayLeds(minVijfVoorHalf,colorMinutes); break;
        case 26: displayLeds(minVijfVoorHalf,colorMinutes); break;
        case 27: displayLeds(minVijfVoorHalf,colorMinutes); break;
        case 28: displayLeds(minVijfVoorHalf,colorMinutes); break;
        case 29: displayLeds(minHalf,colorMinutes); break;
        case 30: displayLeds(minHalf,colorMinutes); break;
        case 31: displayLeds(minHalf,colorMinutes); break;
        case 32: displayLeds(minHalf,colorMinutes); break;
        case 33: displayLeds(minHalf,colorMinutes); break;
        case 34: displayLeds(minVijfOverHalf,colorMinutes); break;
        case 35: displayLeds(minVijfOverHalf,colorMinutes); break;
        case 36: displayLeds(minVijfOverHalf,colorMinutes); break;
        case 37: displayLeds(minVijfOverHalf,colorMinutes); break;
        case 38: displayLeds(minVijfOverHalf,colorMinutes); break;
        case 39: displayLeds(minTienOverHalf,colorMinutes); break;
        case 40: displayLeds(minTienOverHalf,colorMinutes); break;
        case 41: displayLeds(minTienOverHalf,colorMinutes); break;
        case 42: displayLeds(minTienOverHalf,colorMinutes); break;
        case 43: displayLeds(minTienOverHalf,colorMinutes); break;
        case 44: displayLeds(minKwartVoor,colorMinutes); break;
        case 45: displayLeds(minKwartVoor,colorMinutes); break;
        case 46: displayLeds(minKwartVoor,colorMinutes); break;
        case 47: displayLeds(minKwartVoor,colorMinutes); break;
        case 48: displayLeds(minKwartVoor,colorMinutes); break;
        case 49: displayLeds(minTienVoor,colorMinutes); break;
        case 50: displayLeds(minTienVoor,colorMinutes); break;
        case 51: displayLeds(minTienVoor,colorMinutes); break;
        case 52: displayLeds(minTienVoor,colorMinutes); break;
        case 53: displayLeds(minTienVoor,colorMinutes); break;
        case 54: displayLeds(minVijfVoor,colorMinutes); break;
        case 55: displayLeds(minVijfVoor,colorMinutes); break;
        case 56: displayLeds(minVijfVoor,colorMinutes); break;
        case 57: displayLeds(minVijfVoor,colorMinutes); break;
        case 58: displayLeds(minVijfVoor,colorMinutes); break;
        case 59: displayLeds(Uur,colorMinutes); break;
    }

    if (minute > 33) {
        hour++;
    }

    // Hours
    switch (hour) {
        case 1:
        case 13: displayLeds(uurEen, colorHours); break;
        case 2:
        case 14: displayLeds(uurTwee, colorHours); break;
        case 3: 
        case 15: displayLeds(uurDrie, colorHours); break;
        case 4: 
        case 16: displayLeds(uurVier, colorHours); break;
        case 5: 
        case 17: displayLeds(uurVijf, colorHours); break;
        case 6: 
        case 18: displayLeds(uurZes, colorHours); break;
        case 7: 
        case 19: displayLeds(uurZeven, colorHours); break;
        case 8: 
        case 20: displayLeds(uurAcht, colorHours); break;
        case 9: 
        case 21: displayLeds(uurNegen, colorHours); break;
        case 10:
        case 22: displayLeds(uurTien, colorHours); break;
        case 11:
        case 23: displayLeds(uurElf, colorHours); break;
        case 0:
        case 12:
        case 24: displayLeds(uurTwaalf, colorHours); break;
    }
    strip.show();
}

void increaseMinuteTime(byte extraMin) {
    byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;

    // retrieve data from DS3231
    readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
    minute += extraMin;
    minute = minute % 60;

    // setDS3231time(0, minute, hour, dayOfWeek, dayOfMonth, month, year);
    displayTime(minute, hour);
}
void increaseHourTime(byte extraHour) {
    byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;

    // retrieve data from DS3231
    readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
    hour += extraHour;
    hour = hour % 24;

    // setDS3231time(0, minute, hour, dayOfWeek, dayOfMonth, month, year);
    displayTime(minute, hour);
}

void resetTime() {
    byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;

    // retrieve data from DS3231
    readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);

    // setDS3231time(0, 0, 0, dayOfWeek, dayOfMonth, month, year);
}

void loop() {
    /* use timer instead of delay() (blocking) for non-blocking delay */
    unsigned long currentMillis = millis();

    // Update wordclock every second
    if (currentMillis >= 1000 || triggerUpdate) {
        byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
        // retrieve time and date from DS3231
        readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
        #ifdef SERIALON
        //Serial.println(button1State, DEC);
        //Serial.println(button2State, DEC);
        //Serial.println(button3State, DEC);
        //Serial.println(button4State, DEC);
        Serial.print(dayOfWeek, DEC); Serial.print("(zo=1,za=7) "); Serial.print(dayOfMonth, DEC); Serial.print("-"); Serial.print(month, DEC); Serial.print("-"); Serial.print(year, DEC);
        Serial.print(" ");
        Serial.print(hour, DEC); Serial.print(":"); Serial.print(minute, DEC); Serial.print(":"); Serial.print(second, DEC);
        if (DST) {
            Serial.println(" Zomertijd");
        } else {
            Serial.println(" Wintertijd");
        }
        #endif

        // check if we have to remove DST
        if (DST && dayOfWeek == 1 && month == 10 && dayOfMonth >= 25 && dayOfMonth <=31 && hour == 3) {
          increaseHourTime(-1);
          DST = false;
        }
        // check if we have to add DST
        if (!DST && dayOfWeek == 1 && month == 3 && dayOfMonth >= 25 && dayOfMonth <=31 && hour == 2)
        {
          increaseHourTime(1);
          DST = true;
        }
        
            displayTime(minute, hour); // display the real-time clock data on the Serial Monitor and Wordclock

        triggerUpdate = false;
    }
}
